@@ -14,8 +14,8 @@
 // Outputs <= input value
 
 use crate::{
-    Dbc, DbcContent, DbcContentHash, DbcTransaction, Error, Hash, KeyCache, KeyManager,
-    NodeSignature, PublicKeySet, Result,
+    Dbc, DbcContent, DbcContentHash, DbcTransaction, Error, Hash, KeyManager, NodeSignature,
+    PublicKeySet, Result,
 };
 use serde::{Deserialize, Serialize};
 use std::{
@@ -56,9 +56,9 @@ impl ReissueTransaction {
         }
     }
 
-    pub fn validate(&self, key_cache: &KeyCache) -> Result<()> {
+    pub fn validate<K: KeyManager>(&self, verifier: &K) -> Result<()> {
         self.validate_balance()?;
-        self.validate_input_dbcs(key_cache)?;
+        self.validate_input_dbcs(verifier)?;
         self.validate_outputs()?;
         Ok(())
     }
@@ -74,13 +74,13 @@ impl ReissueTransaction {
         }
     }
 
-    fn validate_input_dbcs(&self, key_cache: &KeyCache) -> Result<()> {
+    fn validate_input_dbcs<K: KeyManager>(&self, verifier: &K) -> Result<()> {
         if self.inputs.is_empty() {
             return Err(Error::TransactionMustHaveAnInput);
         }
 
         for input in self.inputs.iter() {
-            input.confirm_valid(key_cache)?;
+            input.confirm_valid(verifier)?;
         }
 
         Ok(())
@@ -123,15 +123,18 @@ pub struct ReissueRequest {
 }
 
 #[derive(Debug, Clone, Serialize, Deserialize)]
-pub struct Mint {
-    pub(crate) key_mgr: KeyManager,
+pub struct Mint<K>
+where
+    K: KeyManager,
+{
+    pub(crate) key_manager: K,
     pub spendbook: SpendBook,
 }
 
-impl Mint {
-    pub fn new(key_mgr: KeyManager) -> Self {
+impl<K: KeyManager> Mint<K> {
+    pub fn new(key_manager: K) -> Self {
         Self {
-            key_mgr,
+            key_manager,
             spendbook: Default::default(),
         }
     }
@@ -140,14 +143,15 @@ impl Mint {
         &mut self,
         amount: u64,
     ) -> Result<(DbcContent, DbcTransaction, (PublicKeySet, NodeSignature))> {
-        self.key_mgr.verify_we_are_a_genesis_node()?;
-
         let parents = BTreeSet::from_iter(vec![GENESIS_DBC_INPUT]);
         let content = DbcContent::new(
             parents,
             amount,
             0,
-            self.key_mgr.public_key_set().public_key(),
+            self.key_manager
+                .public_key_set()
+                .map_err(|e| Error::Signing(e.to_string()))?
+                .public_key(),
         );
         let transaction = DbcTransaction {
             inputs: BTreeSet::from_iter(vec![GENESIS_DBC_INPUT]),
@@ -160,12 +164,20 @@ impl Mint {
         }
 
         self.spendbook.log(GENESIS_DBC_INPUT, transaction.clone());
-        let transaction_sig = self.key_mgr.sign(&transaction.hash());
+        let transaction_sig = self
+            .key_manager
+            .sign(&transaction.hash())
+            .map_err(|e| Error::Signing(e.to_string()))?;
 
         Ok((
             content,
             transaction,
-            (self.key_mgr.public_key_set(), transaction_sig),
+            (
+                self.key_manager
+                    .public_key_set()
+                    .map_err(|e| Error::Signing(e.to_string()))?,
+                transaction_sig,
+            ),
         ))
     }
 
@@ -173,8 +185,8 @@ impl Mint {
         self.spendbook.lookup(&dbc_hash).is_some()
     }
 
-    pub fn key_cache(&self) -> &KeyCache {
-        self.key_mgr.key_cache()
+    pub fn key_manager(&self) -> &K {
+        &self.key_manager
     }
 
     pub fn reissue(
@@ -182,7 +194,7 @@ impl Mint {
         reissue_req: ReissueRequest,
         inputs_belonging_to_mint: BTreeSet<DbcContentHash>,
     ) -> Result<(DbcTransaction, MintSignatures)> {
-        reissue_req.transaction.validate(self.key_cache())?;
+        reissue_req.transaction.validate(self.key_manager())?;
         let transaction = reissue_req.transaction.blinded();
         let transaction_hash = transaction.hash();
 
@@ -204,7 +216,7 @@ impl Mint {
         for input in inputs_belonging_to_mint.iter() {
             if let Some(transaction) = self.spendbook.lookup(&input).cloned() {
                 // This input has already been spent, return the spend transaction to the user
-                let transaction_sigs = self.sign_transaction(&transaction);
+                let transaction_sigs = self.sign_transaction(&transaction)?;
                 return Err(Error::DbcAlreadySpent {
                     transaction,
                     transaction_sigs,
@@ -212,7 +224,7 @@ impl Mint {
             }
         }
 
-        let transaction_sigs = self.sign_transaction(&transaction);
+        let transaction_sigs = self.sign_transaction(&transaction)?;
 
         for input in reissue_req
             .transaction
@@ -229,15 +241,23 @@ impl Mint {
     fn sign_transaction(
         &self,
         transaction: &DbcTransaction,
-    ) -> BTreeMap<DbcContentHash, (PublicKeySet, NodeSignature)> {
-        let sig = self.key_mgr.sign(&transaction.hash());
+    ) -> Result<BTreeMap<DbcContentHash, (PublicKeySet, NodeSignature)>> {
+        let sig = self
+            .key_manager
+            .sign(&transaction.hash())
+            .map_err(|e| Error::Signing(e.to_string()))?;
 
-        transaction
+        Ok(transaction
             .inputs
             .iter()
             .copied()
-            .zip(std::iter::repeat((self.key_mgr.public_key_set(), sig)))
-            .collect()
+            .zip(std::iter::repeat((
+                self.key_manager
+                    .public_key_set()
+                    .map_err(|e| Error::Signing(e.to_string()))?,
+                sig,
+            )))
+            .collect())
     }
 
     // Used in testing / benchmarking
@@ -256,18 +276,24 @@ mod tests {
     use super::*;
     use quickcheck_macros::quickcheck;
 
-    use crate::tests::{TinyInt, TinyVec};
+    use crate::{
+        tests::{TinyInt, TinyVec},
+        SimpleKeyManager, SimpleSigner,
+    };
 
     #[quickcheck]
     fn prop_genesis() {
         let genesis_owner = crate::bls_dkg_id();
         let genesis_key = genesis_owner.public_key_set.public_key();
 
-        let mut genesis_node = Mint::new(KeyManager::new(
-            genesis_owner.public_key_set,
-            (0, genesis_owner.secret_key_share),
-            genesis_key,
-        ));
+        let key_manager = SimpleKeyManager::new(
+            SimpleSigner::new(
+                genesis_owner.public_key_set.clone(),
+                (0, genesis_owner.secret_key_share.clone()),
+            ),
+            genesis_owner.public_key_set.public_key(),
+        );
+        let mut genesis_node = Mint::new(key_manager);
 
         let (gen_dbc_content, gen_dbc_trans, (gen_key_set, gen_node_sig)) =
             genesis_node.issue_genesis_dbc(1000).unwrap();
@@ -286,7 +312,7 @@ mod tests {
         };
 
         assert_eq!(genesis_dbc.content.amount, 1000);
-        let validation = genesis_dbc.confirm_valid(&genesis_node.key_cache());
+        let validation = genesis_dbc.confirm_valid(genesis_node.key_manager());
         assert!(validation.is_ok());
     }
 
@@ -297,11 +323,14 @@ mod tests {
 
         let genesis_owner = crate::bls_dkg_id();
         let genesis_key = genesis_owner.public_key_set.public_key();
-        let mut genesis_node = Mint::new(KeyManager::new(
-            genesis_owner.public_key_set.clone(),
-            (0, genesis_owner.secret_key_share.clone()),
-            genesis_key,
-        ));
+        let key_manager = SimpleKeyManager::new(
+            SimpleSigner::new(
+                genesis_owner.public_key_set.clone(),
+                (0, genesis_owner.secret_key_share.clone()),
+            ),
+            genesis_owner.public_key_set.public_key(),
+        );
+        let mut genesis_node = Mint::new(key_manager.clone());
 
         let (gen_dbc_content, gen_dbc_trans, (gen_key_set, gen_node_sig)) =
             genesis_node.issue_genesis_dbc(output_amount).unwrap();
@@ -385,11 +414,10 @@ mod tests {
                 }
             }));
 
-        let key_cache = KeyCache::from(vec![genesis_key]);
         for dbc in output_dbcs.iter() {
             let expected_amount: u64 = output_amounts[dbc.content.output_number as usize];
             assert_eq!(dbc.amount(), expected_amount);
-            assert!(dbc.confirm_valid(&key_cache).is_ok());
+            assert!(dbc.confirm_valid(&key_manager).is_ok());
         }
 
         assert_eq!(
@@ -399,14 +427,17 @@ mod tests {
     }
 
     #[test]
-    fn test_double_spend_protection() {
+    fn test_double_spend_protection() -> Result<()> {
         let genesis_owner = crate::bls_dkg_id();
         let genesis_key = genesis_owner.public_key_set.public_key();
-        let mut genesis_node = Mint::new(KeyManager::new(
-            genesis_owner.public_key_set,
-            (0, genesis_owner.secret_key_share),
-            genesis_key,
-        ));
+        let key_manager = SimpleKeyManager::new(
+            SimpleSigner::new(
+                genesis_owner.public_key_set.clone(),
+                (0, genesis_owner.secret_key_share.clone()),
+            ),
+            genesis_owner.public_key_set.public_key(),
+        );
+        let mut genesis_node = Mint::new(key_manager);
 
         let (gen_dbc_content, gen_dbc_trans, (gen_key_set, gen_node_sig)) =
             genesis_node.issue_genesis_dbc(1000).unwrap();
@@ -436,11 +467,13 @@ mod tests {
             )]),
         };
 
-        let sig_share = genesis_node.key_mgr.sign(&transaction.blinded().hash());
+        let sig_share = genesis_node
+            .key_manager
+            .sign(&transaction.blinded().hash())?;
 
         let sig = genesis_node
-            .key_mgr
-            .public_key_set()
+            .key_manager
+            .public_key_set()?
             .combine_signatures(vec![sig_share.threshold_crypto()])
             .unwrap();
 
@@ -448,7 +481,7 @@ mod tests {
             transaction,
             input_ownership_proofs: HashMap::from_iter(vec![(
                 genesis_dbc.name(),
-                (genesis_node.key_mgr.public_key_set().public_key(), sig),
+                (genesis_node.key_manager.public_key_set()?.public_key(), sig),
             )]),
         };
 
@@ -467,12 +500,12 @@ mod tests {
         };
 
         let node_share = genesis_node
-            .key_mgr
-            .sign(&double_spend_transaction.blinded().hash());
+            .key_manager
+            .sign(&double_spend_transaction.blinded().hash())?;
 
         let sig = genesis_node
-            .key_mgr
-            .public_key_set()
+            .key_manager
+            .public_key_set()?
             .combine_signatures(vec![node_share.threshold_crypto()])
             .unwrap();
 
@@ -480,7 +513,7 @@ mod tests {
             transaction: double_spend_transaction,
             input_ownership_proofs: HashMap::from_iter(vec![(
                 genesis_dbc.name(),
-                (genesis_node.key_mgr.public_key_set().public_key(), sig),
+                (genesis_node.key_manager.public_key_set()?.public_key(), sig),
             )]),
         };
 
@@ -491,6 +524,8 @@ mod tests {
             res,
             Err(Error::DbcAlreadySpent { transaction, transaction_sigs }) if transaction == t && transaction_sigs == s
         ));
+
+        Ok(())
     }
 
     #[quickcheck]
@@ -529,19 +564,23 @@ mod tests {
 
         let genesis_owner = crate::bls_dkg_id();
         let genesis_key = genesis_owner.public_key_set.public_key();
-        let mut genesis_node = Mint::new(KeyManager::new(
-            genesis_owner.public_key_set,
-            (0, genesis_owner.secret_key_share),
-            genesis_key,
-        ));
+        let key_manager = SimpleKeyManager::new(
+            SimpleSigner::new(
+                genesis_owner.public_key_set.clone(),
+                (0, genesis_owner.secret_key_share.clone()),
+            ),
+            genesis_owner.public_key_set.public_key(),
+        );
+        let mut genesis_node = Mint::new(key_manager);
 
         let genesis_amount: u64 = input_amounts.iter().sum();
         let (gen_dbc_content, gen_dbc_trans, (_gen_key, gen_node_sig)) =
             genesis_node.issue_genesis_dbc(genesis_amount).unwrap();
 
         let genesis_sig = genesis_node
-            .key_mgr
+            .key_manager
             .public_key_set()
+            .unwrap()
             .combine_signatures(vec![gen_node_sig.threshold_crypto()])
             .unwrap();
 
@@ -579,16 +618,25 @@ mod tests {
             input_ownership_proofs: HashMap::default(),
         };
         let sig_share = genesis_node
-            .key_mgr
-            .sign(&reissue_req.transaction.blinded().hash());
+            .key_manager
+            .sign(&reissue_req.transaction.blinded().hash())
+            .unwrap();
         let sig = genesis_node
-            .key_mgr
+            .key_manager
             .public_key_set()
+            .unwrap()
             .combine_signatures(vec![sig_share.threshold_crypto()])
             .unwrap();
         reissue_req.input_ownership_proofs.insert(
             genesis_dbc.name(),
-            (genesis_node.key_mgr.public_key_set().public_key(), sig),
+            (
+                genesis_node
+                    .key_manager
+                    .public_key_set()
+                    .unwrap()
+                    .public_key(),
+                sig,
+            ),
         );
 
         let (transaction, transaction_sigs) =
@@ -744,7 +792,7 @@ mod tests {
                 }));
 
                 for dbc in output_dbcs.iter() {
-                    let dbc_confirm_result = dbc.confirm_valid(&KeyCache::from(vec![genesis_key]));
+                    let dbc_confirm_result = dbc.confirm_valid(&genesis_node.key_manager);
                     println!("DBC confirm result {:?}", dbc_confirm_result);
                     assert!(dbc_confirm_result.is_ok());
                 }
@@ -800,12 +848,14 @@ mod tests {
     #[test]
     fn test_inputs_are_validated() {
         let genesis_owner = crate::bls_dkg_id();
-        let genesis_key = genesis_owner.public_key_set.public_key();
-        let mut genesis_node = Mint::new(KeyManager::new(
-            genesis_owner.public_key_set,
-            (0, genesis_owner.secret_key_share),
-            genesis_key,
-        ));
+        let key_manager = SimpleKeyManager::new(
+            SimpleSigner::new(
+                genesis_owner.public_key_set.clone(),
+                (0, genesis_owner.secret_key_share.clone()),
+            ),
+            genesis_owner.public_key_set.public_key(),
+        );
+        let mut genesis_node = Mint::new(key_manager);
 
         let input_owner = crate::bls_dkg_id();
         let input_content = DbcContent::new(
